/******************************************************************************
*     $Date: 2005/06/15 21:52:25 $
*     $RCSfile: usb_printer.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include <sysace_stdio.h>
#include "xparameters.h"
#include "xuartns550_l.h"
#include "xio.h"
#include "sleep.h"
#include "testpage.h"

// UART Constants
#define UART_BASEADDR  XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK     XPAR_XUARTNS550_CLOCK_HZ
#define UART_BAUDRATE  9600

// USB GPIO Register
#define USB_GPIO       0x9000000C
#define USB_RESET      0x00002000

// USB HPI Registers
#define USB_DATA       0xA5000000
#define USB_MAILBOX    0xA5000004
#define USB_ADDRESS    0xA5000008
#define USB_STATUS     0xA500000C

// USB HPI Addresses
#define COMM_CODE_ADDR 0x01BC

// USB HPI Commands
#define COMM_JUMP2CODE 0xCE00
#define COMM_RESET     0xFA50

// USB User Code Parameters
#define USB_BASE_ADDR  0x04A4
#define USB_BIN_FILE   "a:\\printer.bin"

/* writes a value to the USB chip */
void usb_write(Xuint16 addr, Xuint16 data)
{
  XIo_Out16(USB_ADDRESS, addr);
  XIo_Out16(USB_DATA, data);
}

/* reads a value from the USB chip */
Xuint16 usb_read(Xuint16 addr)
{
  XIo_Out16(USB_ADDRESS, addr);
  return XIo_In16(USB_DATA);
}

/* writes a block of data to the USB chip */
void usb_write_buffer(Xuint16 addr, unsigned char *buffer, Xuint16 length)
{
  int i;

  XIo_Out16(USB_ADDRESS, addr);
  for (i = 0; i < length; i+=2) {
    XIo_Out16(USB_DATA, (buffer[i+1] << 8) | buffer[i]);
  }
}

/* opens and reads from CF a bin file for USB and puts it into the USB chip through HPI */
int read_bin(char FileName[], int baseaddr)
{
  Xuint8 readBuffer[1024];
  SYSACE_FILE *infile;
  int i, numread;

  infile = sysace_fopen(FileName, "r");
  
  if(infile) {
    xil_printf("Loading executable file for USB controller chip.\r\n\r\n");

    XIo_Out16(USB_ADDRESS, baseaddr);

    numread = sysace_fread(readBuffer, 1, 1024, infile);

    while (numread > 0) {
      for (i = 0; i < numread; i+=2) {
	XIo_Out16(USB_DATA, (readBuffer[i+1] << 8) | readBuffer[i]);
      }
      numread = sysace_fread(readBuffer, 1, 1024, infile);
    }

    sysace_fclose(infile);
    return 1;
  } else {
    return 0;
  }
}

/* resets USB chip */
void usb_reset()
{
  XIo_Out16(USB_MAILBOX, COMM_RESET);
}

/* sets PC of USB chip */
void set_pc(Xuint16 addr)
{
  Xuint16 temp;

  // Set COMM_CODE_ADDR to addr
  usb_write(COMM_CODE_ADDR, addr);

  // Set MailBox to execute command
  XIo_Out16(USB_MAILBOX, COMM_JUMP2CODE);
}

/* reads USB chip's mailbox register */
Xuint16 read_mailbox() 
{
  Xuint16 status, temp;

  temp = 0;

  status = XIo_In16(USB_STATUS);

  if (status & 0x001) {
    temp = XIo_In16(USB_MAILBOX);
    xil_printf("Mailbox: %x\r\n", temp);
  }

  return temp;
}

int main()
{
  int temp, timer = 0;
  Xuint16 addr = 0;

  unsigned char *buffer = testpage;
  Xuint16 length = sizeof(testpage);

  /* setup 16550 Uart */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);

  /* hard reset usb */
  temp = XIo_In32(USB_GPIO);
  XIo_Out32(USB_GPIO, temp | USB_RESET);
  XIo_Out32(USB_GPIO, temp & 0xFFFFDFFF);

  xil_printf("\r\nUSB Printer Demo. Prints \"Hello World!\" to USB Printer.\r\n");
  xil_printf("Make sure you have a CF card installed in the ML40x board with the file\r\n");
  xil_printf("<ref design install dir>/sw/standalone/usb_printer/bin/printer.bin\r\n");
  xil_printf("copied into the CF card root directory.\r\n");
  xil_printf("\r\nNote: this program was tested only on Epson Stylus C60 Series printers.\r\n");

  /* read USB binary file from CF */
  if (read_bin(USB_BIN_FILE, USB_BASE_ADDR) == 0) {
    xil_printf("ERROR - Executable file for USB chip not found on Compact Flash.\r\n\r\n");
    return 0;
  }

  /* soft reset usb */
  usb_reset();
  while (XIo_In16(USB_MAILBOX) != 0x0FED) {
    usleep(1);
    timer++;
    if (timer >= 1000) {
      xil_printf("ERROR - USB Chip stuck in reset.\r\n\r\n");
      return 0;
    }
  }

  /* begin execution of new code */
  set_pc(USB_BASE_ADDR);
  while (read_mailbox() != 0x0FED) ;

  /* get buffer addr from USB */
  while ((addr = read_mailbox()) == 0) ;

  /* write buffer */
  usb_write_buffer(addr, buffer, length);

  /* signal USB to begin transfer */
  XIo_Out16(USB_MAILBOX, length);

  /* check USB for updated status (debug only) */
  while (1) {
    read_mailbox();
  }

  return 0;
}
